/**
 * \file
 *
 * \author Martin Stejskal
 *
 * \brief Framebuffer for RGB LED cube 5x5x5
 *
 * Framebuffer display once image on LED cube. As parameter of framebuffer is\n
 * pointer to image in SRAM memory. Framebuffer disable global interrupt\n
 * however enable it when is needed some delay, so in this "time spaces" MCU\n
 * can to other stuff.
 *
*/


// Trick for multiply include this file
#ifndef _framebuffer_library_
#define _framebuffer_library_

/*-----------------------------------------*
 |               Includes                  |
 *-----------------------------------------*/
#include <avr/io.h>             // Defined I/O
#include <avr/interrupt.h>      // Interrupts
#include <inttypes.h>           // Data types
#include "settings.h"           // Symbolic names for pins and other settings
#include <util/delay.h>
#include "bit_operations.h"     // Basic operations with pins
/*-----------------------------------------*
 |               Macros                    |
 *-----------------------------------------*/
/**
 * \brief Turn off all LED in WALL0
 */
#define framebuffer_wall0_void          i_fb_data[0][0] = i_fb_data[0][1] =\
  i_fb_data[0][2] = i_fb_data[0][3] = i_fb_data[0][4] = 0;

/**
 * \brief Turn off all LED in WALL1
 */
#define framebuffer_wall1_void          i_fb_data[1][0] = i_fb_data[1][1] =\
  i_fb_data[1][2] = i_fb_data[1][3] = i_fb_data[1][4] = 0;

/**
 * \brief Turn off all LED in WALL2
 */
#define framebuffer_wall2_void          i_fb_data[2][0] = i_fb_data[2][1] =\
  i_fb_data[2][2] = i_fb_data[2][3] = i_fb_data[2][4] = 0;

/**
 * \brief Turn off all LED in WALL3
 */
#define framebuffer_wall3_void          i_fb_data[3][0] = i_fb_data[3][1] =\
  i_fb_data[3][2] = i_fb_data[3][3] = i_fb_data[3][4] = 0;

/**
 * \brief Turn off all LED in WALL4
 */
#define framebuffer_wall4_void          i_fb_data[4][0] = i_fb_data[4][1] =\
  i_fb_data[4][2] = i_fb_data[4][3] = i_fb_data[4][4] = 0;




/**
 * \brief Turn on all LED in WALL0
 */
#define framebuffer_wall0_ones          i_fb_data[0][0] = i_fb_data[0][1] =\
  i_fb_data[0][2] = i_fb_data[0][3] = i_fb_data[0][4] = 0x7FFF;

/**
 * \brief Turn on all LED in WALL1
 */
#define framebuffer_wall1_ones          i_fb_data[1][0] = i_fb_data[1][1] =\
  i_fb_data[1][2] = i_fb_data[1][3] = i_fb_data[1][4] = 0x7FFF;

/**
 * \brief Turn on all LED in WALL2
 */
#define framebuffer_wall2_ones          i_fb_data[2][0] = i_fb_data[2][1] =\
  i_fb_data[2][2] = i_fb_data[2][3] = i_fb_data[2][4] = 0x7FFF;

/**
 * \brief Turn on all LED in WALL3
 */
#define framebuffer_wall3_ones          i_fb_data[3][0] = i_fb_data[3][1] =\
  i_fb_data[3][2] = i_fb_data[3][3] = i_fb_data[3][4] = 0x7FFF;

/**
 * \brief Turn on all LED in WALL4
 */
#define framebuffer_wall4_ones          i_fb_data[4][0] = i_fb_data[4][1] =\
  i_fb_data[4][2] = i_fb_data[4][3] = i_fb_data[4][4] = 0x7FFF;



/**
 * \brief Copy data from WALL0 to WALL1
 */
#define framebuffer_copy_wall0_to_wall1         \
  i_fb_data[1][0] = i_fb_data[0][0];\
  i_fb_data[1][1] = i_fb_data[0][1];\
  i_fb_data[1][2] = i_fb_data[0][2];\
  i_fb_data[1][3] = i_fb_data[0][3];\
  i_fb_data[1][4] = i_fb_data[0][4];

/**
 * \brief Copy data from WALL1 to WALL2
 */
#define framebuffer_copy_wall1_to_wall2         \
  i_fb_data[2][0] = i_fb_data[1][0];\
  i_fb_data[2][1] = i_fb_data[1][1];\
  i_fb_data[2][2] = i_fb_data[1][2];\
  i_fb_data[2][3] = i_fb_data[1][3];\
  i_fb_data[2][4] = i_fb_data[1][4];

/**
 * \brief Copy data from WALL2 to WALL3
 */
#define framebuffer_copy_wall2_to_wall3         \
  i_fb_data[3][0] = i_fb_data[2][0];\
  i_fb_data[3][1] = i_fb_data[2][1];\
  i_fb_data[3][2] = i_fb_data[2][2];\
  i_fb_data[3][3] = i_fb_data[2][3];\
  i_fb_data[3][4] = i_fb_data[2][4];

/**
 * \brief Copy data from WALL3 to WALL4
 */
#define framebuffer_copy_wall3_to_wall4         \
  i_fb_data[4][0] = i_fb_data[3][0];\
  i_fb_data[4][1] = i_fb_data[3][1];\
  i_fb_data[4][2] = i_fb_data[3][2];\
  i_fb_data[4][3] = i_fb_data[3][3];\
  i_fb_data[4][4] = i_fb_data[3][4];


/*-----------------------------------------*
 |         Function prototypes             |
 *-----------------------------------------*/
/**
 * \brief Frame buffer use pointer to array with data, witch represents one 3D
 * image
 *
 * This function display data pointed to array \b p_fb_data\b [\b ]\b [\b ] to
 * LED cube. Whole function must run in infinite loop, else just once image
 * will be showed (for very short time). And because of multiplex, LEDs have to
 * been "refreshed"
 *
 * \param p_fb_data Pointer to frame buffer. First index is for "wall",
 * second is for "column". Size of every "column" is 16 bits. MSB bit (highest
 * LED in "column") is always "first". LSB bit (lowest one) is "last".\n
 *     |MSB|           |LSB|MSB|           |LSB|MSB|           |LSB|\n
 * +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+\n
 * | X | R | R | R | R | R | G | G | G | G | G | B | B | B | B | B |\n
 * +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+\n
 */
void framebuffer(volatile uint16_t (*p_fb_data)[5][5]);

/**
 * \brief Set I/O for framebuffer to defaults
 *
 * Set only pins, that framebuffer controls
 */
void init_framebuffer(void);


/**
 * \brief According parameter set active "wall".
 *
 * If input parameter is out of HW limit, then none "wall" is activated
 *
 * \param i_wall_number Wall number - 0~5
 */
void set_active_wall( uint8_t i_wall_number );


/**
 * \brief According parameter set active "column".
 *
 * If input parameter is out of HW limit, then none "column" is activated
 *
 * \param i_column_number Column number 0~5
 */
void set_active_column( uint8_t i_column_number );


/**
 * \brief Load bits stored in i_data_r to bus with \b red LEDs
 *
 * LSB is represent lowest red diode in column
 *
 * \param i_data_r Binary data for red color
 */
void load_DATA_RED_on_bus(   uint8_t i_data_r );


/**
 * \brief Load bits stored in i_data_g to bus with \b green LEDs
 *
 * LSB is represent lowest green diode in column
 *
 * \param i_data_g Binary data for green color
 */
void load_DATA_GREEN_on_bus( uint8_t i_data_g );



/**
 * \brief Load bits stored in i_data_b to bus with \b blue LEDs
 *
 * LSB is represent lowest blue diode in column
 *
 * \param i_data_b Binary data for blue color
 */
void load_DATA_BLUE_on_bus(  uint8_t i_data_b );


#endif
