/**
 * \file
 *
 * \author Martin Stejskal
 *
 * \brief Complex settings AVR for project
 *
 * In this file is defined speed of CPU, names for pins and PORTs and so on
*/

// Trick for multiply include this file
#ifndef _settings_library_
#define _settings_library_

/**
 * \brief MCU speed
 *
 * Expected speed of oscillator for AVR.
 */
#define F_CPU           16000000UL

/**
 * \brief Time space (in us) between "two columns"
 *
 * ----- time ----->\n
 * \< COLUMN0 active \> \< time delay \> \< COLUMN1 active \>
 * \< time delay \> \< COLUMN2 active \> ....
 */
#define column_delay    500

/**
 * \brief Value for animation timer to get approximately 10ms delay
 *
 * Delay is calculated for prescaller in function init_animation_counter.\n
 * For 16 MHz crystal and prescaller 1024 is this value 156.\n
 * 16 MHz -> 62.5 ns -> 62.5 ns * 1024 -> 64 us /1 timer cycle ->\n
 * -> 10 ms / 64us = 156.25 -> 156
 */
#define animation_delay_timer   156

/**
 * \brief Enable/Disable PWM support
 *
 * If project will be ported on different AVR, there can be problem that\n
 * three PWM channel will not be on same counter. If PWM is disabled, then\n
 * outputs PWM_R*, PWM_G* and PWM_B* will be set to 1. Anyway it is good idea\n
 * to change PCB (when is changed AVR), so it is easier connect "R_GND",\n
 * "G_GND" and "B_GND" direct to GND. Then any code with PWM can be removed.
 * \b Options: \b enabled, \b disabled
 */
#define pwm_support     enabled
#define enabled         1
#define disabled        0

/**
 * \brief Define witch version will be compiled
 *
 * Now it is easy to switch from debug to release code and vice versa :) Debug
 * code can not affect release version.\n
 * \b Options: \b debug, \b release
 * \{
 */
#define version         debug
#define debug           0
#define release         1
/**
 * \}
 */

/**
 * \defgroup io_names I/O symbolic names
 *
 * \brief Symbolic port and pin names
 *
 * To every pin is there defined symbolic name. If this symbolic name is
 * changed here, then is changed by preprocessor "everywhere"
 * \{
 */

/**
 * \ingroup io_names
 * \defgroup data_red Data for red color
 * \defgroup data_green Data for green color
 * \defgroup data_blue Data for blue color
 * \defgroup wall Pins for select wall
 * \defgroup column Pins for select columns
 * \defgroup pwm_rgb Pins connected to hardware PWM on AVR chip
 *  Do not change unless You really know what are You doing!
 * \defgroup keyboard Pins connected to matrix keyboard
 * \defgroup uart UART pins
 * Do not change unless You really know what are You doing!
 */

/**
 * \ingroup data_red
 * \{
 */
#define D_R0_PORT       G
#define D_R0_pin    4
#define D_R1_PORT       G
#define D_R1_pin    3
#define D_R2_PORT       B
#define D_R2_pin    7
#define D_R3_PORT       B
#define D_R3_pin    6
#define D_R4_PORT       B
#define D_R4_pin    5
/**
 * \}
 * \ingroup data_green
 * \{
 */
#define D_G0_PORT       B
#define D_G0_pin    4
#define D_G1_PORT       B
#define D_G1_pin    3
#define D_G2_PORT       B
#define D_G2_pin    2
#define D_G3_PORT       B
#define D_G3_pin    1
#define D_G4_PORT       B
#define D_G4_pin    0
/**
 * \}
 * \ingroup data_blue
 * \{
 */
#define D_B0_PORT       E
#define D_B0_pin    7
#define D_B1_PORT       E
#define D_B1_pin    6
#define D_B2_PORT       A
#define D_B2_pin    5
#define D_B3_PORT       A
#define D_B3_pin    4
#define D_B4_PORT       A
#define D_B4_pin    3
/**
 * \}
 *
 * \ingroup wall
 * \{
 */
#define WALL0_PORT      A
#define WALL0_pin   6
#define WALL1_PORT      A
#define WALL1_pin   7
#define WALL2_PORT      C
#define WALL2_pin   7
#define WALL3_PORT      C
#define WALL3_pin   5
#define WALL4_PORT      C
#define WALL4_pin   3
/**
 * \}
 *
 * \ingroup column
 * \{
 */
#define COLUMN0_PORT    D
#define COLUMN0_pin 5
#define COLUMN1_PORT    D
#define COLUMN1_pin 7
#define COLUMN2_PORT    G
#define COLUMN2_pin 0
#define COLUMN3_PORT    G
#define COLUMN3_pin 1
#define COLUMN4_PORT    C
#define COLUMN4_pin 1
/**
 * \}
 *
 * \ingroup pwm_rgb
 * \{
 */
#define PWM_R_PORT      E
#define PWM_R_pin   3
#define PWM_G_PORT      E
#define PWM_G_pin   4
#define PWM_B_PORT      E
#define PWM_B_pin   5
/**
 * \}
 *
 * \ingroup keyboard
 * \{
 */
#define KBRD_RD0_PORT           F
#define KBRD_RD0_pin        5
#define KBRD_RD1_PORT           F
#define KBRD_RD1_pin        4
#define KBRD_RD2_PORT           F
#define KBRD_RD2_pin        3
#define KBRD_RD3_PORT           F
#define KBRD_RD3_pin        2
#define KBRD_RD4_PORT           F
#define KBRD_RD4_pin        1
#define KBRD_RD5_PORT           F
#define KBRD_RD5_pin        0

#define KBRD_PWR0_PORT          A
#define KBRD_PWR0_pin       2
#define KBRD_PWR1_PORT          A
#define KBRD_PWR1_pin       1
#define KBRD_PWR2_PORT          A
#define KBRD_PWR2_pin       0
#define KBRD_PWR3_PORT          F
#define KBRD_PWR3_pin       7
#define KBRD_PWR4_PORT          F
#define KBRD_PWR4_pin       6
/**
 * \}
 *
 * \ingroup uart
 * \{
 */
#define RX_PC_PORT              D
#define RX_PC_pin           3
#define TX_PC_PORT              D
#define TX_PC_pin           2
/**
 * \}
 */


/**
 * \}
 */

/**
 * \defgroup stat_cfg_anime i_stat_cfg_anime bits
 *
 * \brief Bit names in i_stat_cfg_anime variable
 *
 * This values should not be changed, until you really know what are you doing!
 *
 * \ingroup stat_cfg_anime
 * \{
 */
#define anim_run        7
#define anim_change     6

#define anim_random     5

// Note: anim_cnt* must be "near" to each other. Example: 7,6 or 1,0 and so on
#define anim_cnt1       3
#define anim_cnt0       2

#define anim_SRAM_valid 1
#define anim_read_FLASH 0
/**
 * \}
 */

/**
 * \brief Maximum read address when reading from memory
 */
#define anim_random_max_address_flash   6000

#endif
