/**
 * \file
 *
 * \author Martin Stejskal
 *
 * \brief Main file for RGB LED cube 5x5x5
*/

#include <avr/io.h>             // Definitions I/O
#include <inttypes.h>           // Data types
#include <avr/interrupt.h>      // Interrupts
#include <avr/pgmspace.h>       // For operations with flash memory

#include "settings.h"           // Defined symbolic names, F_CPU, and so on

#include <util/delay.h>         // Delays library

#include "bit_operations.h"     // Basic operations with bits and I/O

#include "framebuffer.h"        // Framebuffer functions for
#include "pwm_rgb.h"            // Driver for PWM

#include "animation_codes.h"    // Command codes in animations

// Animations
#include "anim_snake.h"
//#include "anim_test.h"

/*-----------------------------------------*
 |             Global variables            |
 *-----------------------------------------*/
// Usually needed because of interrupt functions
/**
 * \brief Framebuffer for 1 image, that is displayed
 *
 * Define frame buffer - buffer size: 16bits -> 3 colors x 5 led in\n
 * column -> 5 columns in one wall -> 5x16bits -> 5 walls -> 5x5x16bits\n
 * First index is for "wall" (0~4) and second is for "column" (0~4).\n
 * Must be global and volatile, because it is used also by interrupts.
 */
/*
volatile uint16_t i_fb_data[5][5] =
// Fill with demo data - cube
{
        {0x7C00, 0x4400, 0x4400, 0x4400, 0x7C00},
        {0x0220, 0x0000, 0x0000, 0x0000, 0x0220},
        {0x0220, 0x0000, 0x0000, 0x0000, 0x0220},
        {0x0220, 0x0000, 0x0000, 0x0000, 0x0220},
        {0x001F, 0x0011, 0x0011, 0x0011, 0x001F}
};
*/
/*
volatile uint16_t i_fb_data[5][5] =
// Fill with demo data - white
{
        {0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF},
        {0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF},
        {0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF},
        {0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF},
        {0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF, 0x7FFF}
};
*/
volatile uint16_t i_fb_data[5][5] =
// Fill with demo data - nothing - no LED on
{
        {0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
        {0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
        {0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
        {0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
        {0x0000, 0x0000, 0x0000, 0x0000, 0x0000}
};



/**
 * \brief Status and configuration variable for animation process
 *
 * Bit meaning:\n
 * 7 (MSB) - Animation is running (1 - running ; 0 - not running)\n
 *           Named as: anim_run
 *
 * 6 - request for change animation (pointer is updated)\n
 *     Named as: anim_change
 * 5 - random - If activated, the all command are ignored and just dummy read
 * 4
 * 3 - counter[1] - phase of animation waiting process (0~3 - if 3 -> change\n
 *                  image)\n
 *                  Named as: anim_cnt1\n
 * 2 - counter[0]   Named as: anim_cnt0\n
 * 1 - Data on SRAM valid\n
 *     Named as: anim_SRAM_valid\n
 * 0 - Data are stored in FLASH (if 1, else data are stored in SRAM - 0)\n
 *     Named as: anim_read_FLASH\n
 *
 */
volatile uint8_t i_stat_cfg_anime = 0b00000000;

/**
 * \brief Pointer for animation process
 *
 * To this pointer is stored SRAM or FLASH address, where are data stored.\n
 * 16 bit pointer should be enough.
 */
volatile uint16_t *p_anime = 0;

/**
 * \brief Slowdown factor for animation
 *
 * Every 40 ms is possible load new frame -> 25 fps. However, sometimes it is\n
 * too short time, so it is possible make animation slower. If is set to 1,\n
 * then every 40 ms is load new frame. If is set to 2, then every 80 ms is\n
 * loaded new frame and so on. This value can be changed any time during\n
 * animation (Should be part of animation code).
 *
 */
volatile uint8_t  anime_slowdown_factor = 1;

/**
 * \brief Counter for slowdown factor
 *
 * Internal variable. Value is increased every 40 ms and set to 1 when\n
 * anime_slowdown_factor == anime_slowdown_counter. Should not be changed by\n
 * user.
 */
volatile uint8_t  anime_slowdown_counter = 1;

/*-----------------------------------------*
 |         Function prototypes             |
 *-----------------------------------------*/
void set_io_to_defaults(void);

void init_animation_counter(void);

inline void animation_process_word(uint16_t i_tmp, uint8_t *p_framebuffer_cnt);

/*-----------------------------------------*
 |               Interrupts                |
 *-----------------------------------------*/
/**
 * \brief Function for animation process.
 *
 * Approximately every 10 ms is called this interrupt. Every 40 ms (every\n
 * fourth time) this function increase pointer value and read another image.\n
 * If animation is at the end, then is wait for a "new pointer" and show last\n
 * image in buffer. Use timer2, because interrupt priority is one of the first.
 */
ISR(TIMER2_COMP_vect)
{
  /* Let's find out in witch time phase function is. If in phase 3 (anim_cnt0
   * and anim_cnt1 must be in "1"). It is used masking to get non-shifted value
   */

  /* Save phase to variable. Use masking and then value is shifted, so we can
   * get value 0~3
   */
  uint8_t i_phase = i_stat_cfg_anime &\
      ((1<<anim_cnt0)|(1<<anim_cnt1));   // Get mask
  i_phase = i_phase>>anim_cnt0;         // Shift to left -> now we get 0~3

  // Test if run out 40 ms
  if( i_phase == 3 )
  {
    i_phase = 0;        // Set phase to begin (always if i_phase == 3)

    // Check slowdown factor
    if( anime_slowdown_counter == anime_slowdown_factor )
    {
      // OK, first reset slowdown counter to 1
      anime_slowdown_counter = 1;


      // Check animation status - Run/Stop (masking)
      if( (i_stat_cfg_anime & (1<<anim_run)) != 0 )
      {// Animation run
        // Variable for temporary data
        uint16_t i_tmp = 0;

        // Counter for bytes loaded to framebuffer
        uint8_t i_framebuffer_cnt = 0;


        /* OK, if status is "run", then is needed to know from witch memory
         * data will be loaded. Options are flash and SRAM. So let's find
         * out...
         */
        if( (i_stat_cfg_anime & (1<<anim_read_FLASH)) != 0)
        { // Data are stored in flash memory

          // OK, so fill whole framebuffer
          while( i_framebuffer_cnt < 25 )
          {
            i_tmp = pgm_read_word( p_anime );     // Load word (16 bits)
            p_anime++;                            // Increment pointer by 1

            // if random is enabled
            if( ( ( i_stat_cfg_anime & (1<<anim_random) ) != 0) &&
                // And address is bigger then max....
                ((p_anime)>anim_random_max_address_flash))
            {
              p_anime = 0x00;   // set pointer to begin
            }

            /* And process loaded data. If data were written to framebuffer,
             * then increase "i_framebuffer_cnt" by 1
             */
            animation_process_word( i_tmp, &i_framebuffer_cnt );
          }
        }
        else
        { // Else data are stored in SRAM
          // OK, so fill whole framebuffer
          while( i_framebuffer_cnt < 25)
          {
            i_tmp = *p_anime;                     // Load word (16 bits)
            p_anime++;                            // Increase pointer py 1

            /* And process loaded data. If data were written to framebuffer,
             * then increase "i_framebuffer_cnt" by 1
             */
            animation_process_word( i_tmp,& i_framebuffer_cnt );
          }
        }
      }
      // Else animation Stop -> nothing to do

    }
    else
    {// If anime_slowdown_counter != anime_slowdown_factor
      anime_slowdown_counter++;
    }
  }
  else
  {// If i_phase != 3 -> increase
    i_phase++;          // Increase phase number
  }

  // Use mask to clear counter values
  i_stat_cfg_anime &= ~((1<<anim_cnt1)|(1<<anim_cnt0));

  // Anyway, i_phase has to be written back as updated value
  i_phase = i_phase << anim_cnt0;       // Shift back to left

  // Mask & i_phase value -> i_phase saved
  i_stat_cfg_anime |= ((1<<anim_cnt1)|(1<<anim_cnt0)) & i_phase;
}




/**
 * \brief Main function
 *
 * Initialization of functions\n
 * Show content of i_fb_data in infinite loop\n
 *
 * \return \b 0 if all right, but this should never happened
 */
int main(void)
{
  // First initialize all pins
  set_io_to_defaults(); /** \todo Remove */

  // Initialize framebuffer
  init_framebuffer();

  // Initialize PWM driver
  init_pwm_rgb();

  // Initialize Timer2 for animation process
  init_animation_counter();

  // DEBUG
  i_stat_cfg_anime = 0x81;
  //i_stat_cfg_anime |= (1<<anim_random);
  //anime_slowdown_factor = 7;
  //anime_slowdown_factor = 1;
  p_anime = bin_anim_snake;
  //p_anime = 0x0000;
  // /DEBUG

  while(1)
  {     // Run framebuffer in infinite loop
    framebuffer( &i_fb_data );   // Give pointer to frame buffer
  }

  return(0);
}


/**
 * \brief Set I/O to defaults
 *
 * Prepare all pins to default state. This function first \b set \b all pins\n
 * to \b Hi-Z and then set I/O
 */
void set_io_to_defaults(void)
{
  /* OK, first (just for case) set all PORTx and DDRx to 0x00 -> Hi-Z on all
   * pins. Because of possible port on other AVR devices there are used "ifdef"
   * conditions. If needed, define another
   */
#ifdef PORTA    // If on HW exist this port -> set whole port to Hi-Z
  PORTA = 0x00;
  DDRA  = 0x00;
#endif
#ifdef PORTB    // If on HW exist this port -> set whole port to Hi-Z
  PORTB = 0x00;
  DDRB  = 0x00;
#endif
#ifdef PORTC    // If on HW exist this port -> set whole port to Hi-Z
  PORTC = 0x00;
  DDRC  = 0x00;
#endif
#ifdef PORTD    // If on HW exist this port -> set whole port to Hi-Z
  PORTD = 0x00;
  DDRD  = 0x00;
#endif
#ifdef PORTE    // If on HW exist this port -> set whole port to Hi-Z
  PORTE = 0x00;
  DDRE  = 0x00;
#endif
#ifdef PORTF    // If on HW exist this port -> set whole port to Hi-Z
  PORTF = 0x00;
  DDRF  = 0x00;
#endif
#ifdef PORTG    // If on HW exist this port -> set whole port to Hi-Z
  PORTG = 0x00;
  DDRG  = 0x00;
#endif
#ifdef PORTH    // If on HW exist this port -> set whole port to Hi-Z
  PORTH = 0x00;
  DDRH  = 0x00;
#endif
#ifdef PORTI    // If on HW exist this port -> set whole port to Hi-Z
  PORTI = 0x00;
  DDRI  = 0x00;
#endif

  // Keyboard - KBRD_RDx - all input and enabled pull-up
  io_set_dir_in( KBRD_RD0_PORT, KBRD_RD0_pin );
  io_set_1( KBRD_RD0_PORT, KBRD_RD0_pin );      // Activate pull-up
  io_set_dir_in( KBRD_RD1_PORT, KBRD_RD1_pin );
  io_set_1( KBRD_RD1_PORT, KBRD_RD1_pin );      // Activate pull-up
  io_set_dir_in( KBRD_RD2_PORT, KBRD_RD2_pin );
  io_set_1( KBRD_RD2_PORT, KBRD_RD2_pin );      // Activate pull-up
  io_set_dir_in( KBRD_RD3_PORT, KBRD_RD3_pin );
  io_set_1( KBRD_RD3_PORT, KBRD_RD3_pin );      // Activate pull-up
  io_set_dir_in( KBRD_RD4_PORT, KBRD_RD4_pin );
  io_set_1( KBRD_RD4_PORT, KBRD_RD4_pin );      // Activate pull-up
  io_set_dir_in( KBRD_RD5_PORT, KBRD_RD5_pin );
  io_set_1( KBRD_RD5_PORT, KBRD_RD5_pin );      // Activate pull-up

  // Keyboard - KBRD_PWRx - all as Hi-Z -> set as input without pull-up
  io_set_dir_in( KBRD_PWR0_PORT, KBRD_PWR0_pin );
  io_set_dir_in( KBRD_PWR1_PORT, KBRD_PWR1_pin );
  io_set_dir_in( KBRD_PWR2_PORT, KBRD_PWR2_pin );
  io_set_dir_in( KBRD_PWR3_PORT, KBRD_PWR3_pin );
  io_set_dir_in( KBRD_PWR4_PORT, KBRD_PWR4_pin );

  // UART - PC
  io_set_dir_in(  RX_PC_PORT, RX_PC_pin );
  io_set_dir_out( TX_PC_PORT, TX_PC_pin );
}


/**
 * \brief Initialize Timer2 for interrupt, witch use animation function
 *
 * Set timer2 to CTC mode and set interrupt time to approximately 10 ms
 */
void init_animation_counter(void)
{
  // normal pin mode ; CTC mode ; prescaller 1024 -> 64us / 1 cycle
  TCCR2 = (0<<COM21)|(0<<COM20)|\
          (1<<WGM21)|(0<<WGM20)|\
          (1<<CS22) |(0<<CS21) |(1<<CS20);

  // Fill OCR2 -> approx 10ms
  OCR2 = animation_delay_timer;

  // Enable interrupt if counter counts to value in OCR2
  TIMSK |= (1<<OCIE2);
}



/**
 * \brief Process loaded word to "i_tmp" and if needed save data to\n
 * framebuffer, or proceed special commands
 *
 *
 * @param i_tmp Loaded word (16 bit) from memory (Flash or SRAM)
 *
 * @param i_framebuffer_cnt Counter for framebuffer. It counts how many words\n
 * were written to framebuffer
 */
inline void animation_process_word(uint16_t i_tmp, uint8_t *p_framebuffer_cnt)
{
  // Test for special functions -> first bit is set
  if( (( i_tmp & 0x8000 ) != 0) &&
      (((i_stat_cfg_anime) & (1<<anim_random)) == 0) )
  {/* Special function -> i_tmp contains 2 parts: command number and value ->
    * -> let's split them -> easier for AVR to work with 8bit variables than
    * 16bit
    */
    uint8_t cmd_number = (i_tmp & 0x7F00)>>8;   // Save command number
    uint8_t cmd_value  = (i_tmp & 0x00FF);      // And save value
    switch( cmd_number )
    {
    // Command animation start
    case (cmd_anim_start>>8)& 0x7F :    // From 16 bit number makes cmd number
      anime_slowdown_factor = cmd_value;        // Save slowdown factor
      // Set PWM to max
      pwm_set_r(255);
      pwm_set_g(255);
      pwm_set_b(255);
      break;

    // Command animation stop
    case (cmd_anim_stop>>8)& 0x7F :
      if(cmd_value == 0)        // Test for option "0"
      {
        for(uint8_t i_wall = 4 ; i_wall <5 ; i_wall--)

          for(uint8_t i_column = 4 ; i_column <5 ; i_column--)
          {
            i_fb_data[i_wall][i_column] = 0;
        }
      }
      if(cmd_value == 1)        // Test for option "1"
      {
        // Nothing to do (framebuffer stay same)
      }
      // And set animation to "stop"
      i_stat_cfg_anime = i_stat_cfg_anime & ~(1<<anim_run);     // Anime stop
      *p_framebuffer_cnt = 25;  /* And set counter to 25 -> stop loading data
                                 * to framebuffer.
                                 */
      break;

    // Command Set PWM for RED channel
    case (cmd_set_pwm_r>>8)& 0x7F :
      pwm_set_r( cmd_value ); break;

    // Command Set PWM for GREEN channel
    case (cmd_set_pwm_g>>8)& 0x7F :
      pwm_set_g( cmd_value ); break;

    // Command Set PWM for BLUE channel
    case (cmd_set_pwm_b>>8)& 0x7F :
      pwm_set_b( cmd_value ); break;


    // Parameters for 2D frame (active wall)
    case (cmd_param_2D_frame>>8)& 0x7F :
      // Decide according option
      switch( cmd_value )
      {
      case param_2D_frame_void: // Void 2D frame
        if(*p_framebuffer_cnt < 5 )
        {
          // Power off all LED in WALL0
          framebuffer_wall0_void;
          // And set counter on following wall
          *p_framebuffer_cnt = 5;
        }
        else if(*p_framebuffer_cnt < 10)
        {
          // Power off all LED in WALL1
          framebuffer_wall1_void;
          // And set counter on following wall
          *p_framebuffer_cnt = 10;
        }
        else if(*p_framebuffer_cnt < 15)
        {
          // Power off all LED in WALL2
          framebuffer_wall2_void;
          // And set counter on following wall
          *p_framebuffer_cnt = 15;
        }
        else if(*p_framebuffer_cnt < 20)
        {
          // Power off all LED in WALL3
          framebuffer_wall3_void;
          // And set counter on following wall
          *p_framebuffer_cnt = 20;
        }
        else
        {
          // Power off all LED in WALL4
          framebuffer_wall4_void;
          // And set counter on following wall
          *p_framebuffer_cnt = 25;
        }
        break;  // param_2D_frame_void
      case param_2D_frame_same: // Same 2D frame as previous
        // Just copy data and move on
        if(*p_framebuffer_cnt < 5)
        {// Error - WALL0 has no reference -> clear all data
          framebuffer_wall0_void;
          // And set counter on following wall
          *p_framebuffer_cnt = 5;
        }
        else if(*p_framebuffer_cnt < 10)
        {// Active WALL1 -> copy from WALL0
          framebuffer_copy_wall0_to_wall1;
          // And set counter on following wall
          *p_framebuffer_cnt = 10;
        }
        else if(*p_framebuffer_cnt < 15)
        {// Active WALL2 -> copy from WALL1
          framebuffer_copy_wall1_to_wall2;
          // And set counter on following wall
          *p_framebuffer_cnt = 15;
        }
        else if(*p_framebuffer_cnt < 20 )
        {// Active WALL3 -> copy from WALL2
          framebuffer_copy_wall2_to_wall3;
          // And set counter on following wall
          *p_framebuffer_cnt = 20;
        }
        else
        {// Active WALL4 -> copy from WALL3
          framebuffer_copy_wall3_to_wall4;
          // And set counter on following wall
          *p_framebuffer_cnt = 25;
        }
        break;  // param_2D_frame_same
      case param_2D_frame_ones:        // Whole 2D frame is on
      if(*p_framebuffer_cnt < 5 )
      {
        // Turn on all LEDs in WALL0
        framebuffer_wall0_ones;
        // And set counter on following wall
        *p_framebuffer_cnt = 5;
      }
      else if(*p_framebuffer_cnt < 10)
      {
        // Turn on all LEDs in WALL1
        framebuffer_wall1_ones;
        // And set counter on following wall
        *p_framebuffer_cnt = 10;
      }
      else if(*p_framebuffer_cnt < 15)
      {
        // Turn on all LEDs in WALL2
        framebuffer_wall2_ones;
        // And set counter on following wall
        *p_framebuffer_cnt = 15;
      }
      else if(*p_framebuffer_cnt < 20)
      {
        // Turn on all LEDs in WALL3
        framebuffer_wall3_ones;
        // And set counter on following wall
        *p_framebuffer_cnt = 20;
      }
      else
      {
        // Turn on all LEDs in WALL4
        framebuffer_wall4_ones;
        // And set counter on following wall
        *p_framebuffer_cnt = 25;
      }
        break;  // param_2D_frame_ones
      }
    break;// For case: cmd_param_2D_frame

    case (cmd_param_3D_frame>>8)& 0x7F :
    // Ok, there will be some options
      switch(cmd_value)
      {
      case param_3D_frame_void: // Set all segments to 0
        framebuffer_wall0_void;
        framebuffer_wall1_void;
        framebuffer_wall2_void;
        framebuffer_wall3_void;
        framebuffer_wall4_void;
        break;  // param_3D_frame_void

      case param_3D_frame_same: /* Framebuffer same -> do not change
                                 * framebuffer -> just change p_framebuffer_cnt
                                 */
        *p_framebuffer_cnt = 25;
        break;    // param_3D_frame_same
      case param_3D_frame_ones: // Turn on all LEDs
        framebuffer_wall0_ones;
        framebuffer_wall1_ones;
        framebuffer_wall2_ones;
        framebuffer_wall3_ones;
        framebuffer_wall4_ones;
        break;    // param_3D_frame_ones
      }
    break;      // cmd_param_3D_frame
    }

    /**
     * \todo Another special functions
     */
  }
  else
  {// Else ordinary data

    /* Depend on i_framebuffer_cnt store data on different position.
     * Used switch because of speed. Because framebuffer is 16 bit,
     * data must be written in "two cycles"
     */
    switch(*p_framebuffer_cnt)
    {
    case 0:  i_fb_data[0][0] = i_tmp; break;
    case 1:  i_fb_data[0][1] = i_tmp; break;
    case 2:  i_fb_data[0][2] = i_tmp; break;
    case 3:  i_fb_data[0][3] = i_tmp; break;
    case 4:  i_fb_data[0][4] = i_tmp; break;
    case 5:  i_fb_data[1][0] = i_tmp; break;
    case 6:  i_fb_data[1][1] = i_tmp; break;
    case 7:  i_fb_data[1][2] = i_tmp; break;
    case 8:  i_fb_data[1][3] = i_tmp; break;
    case 9:  i_fb_data[1][4] = i_tmp; break;
    case 10: i_fb_data[2][0] = i_tmp; break;
    case 11: i_fb_data[2][1] = i_tmp; break;
    case 12: i_fb_data[2][2] = i_tmp; break;
    case 13: i_fb_data[2][3] = i_tmp; break;
    case 14: i_fb_data[2][4] = i_tmp; break;
    case 15: i_fb_data[3][0] = i_tmp; break;
    case 16: i_fb_data[3][1] = i_tmp; break;
    case 17: i_fb_data[3][2] = i_tmp; break;
    case 18: i_fb_data[3][3] = i_tmp; break;
    case 19: i_fb_data[3][4] = i_tmp; break;
    case 20: i_fb_data[4][0] = i_tmp; break;
    case 21: i_fb_data[4][1] = i_tmp; break;
    case 22: i_fb_data[4][2] = i_tmp; break;
    case 23: i_fb_data[4][3] = i_tmp; break;
    case 24: i_fb_data[4][4] = i_tmp; break;
    }

    (*p_framebuffer_cnt)++;        // Increase counter by 1
  }

}
